/****************************************************************************/
/*                   Thinking Outside the In-Memory Box                     */
/*                            PASS Summit 2015                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                       Database Creation Script                           */
/****************************************************************************/

set noexec off
go

use master
go

if convert(int,
			left(
				convert(nvarchar(128), serverproperty('ProductVersion')),
				charindex('.',convert(nvarchar(128), serverproperty('ProductVersion'))) - 1
			)
	) < 12 
begin
	raiserror('You should have SQL Server 2014 to execute this script',16,1) with nowait
	set noexec on
end
go

if convert(int, serverproperty('EngineEdition')) != 3 or charindex('X64',@@Version) = 0
begin
	raiserror('That script requires 64-Bit Enterprise Edition of SQL Server to run',16,1)
	set noexec on
end
go

if exists
(
	select * from sys.databases where name = 'InMemoryBoxDemo'
)
begin
	raiserror('Database InMemoryBoxDemo already created',16,1)
	set noexec on
end
go



declare
	@dataPath nvarchar(512) = convert(nvarchar(512),serverproperty('InstanceDefaultDataPath'))
	,@logPath nvarchar(512) = convert(nvarchar(512),serverproperty('InstanceDefaultLogPath'))

/*** REPLACE IF YOU WANT TO STORE IN-MEMORY OLTP FILES IN THE DIFFERENT PLACE ***/
declare
	@HKPath nvarchar(512) = @dataPath + 'InMemoryBoxDemo_HKData'
	
declare
	@SQL nvarchar(max)

select @SQL = 
N'create database [InMemoryBoxDemo] on 
primary (name=N''InMemoryBoxDemo'', filename=N''' + @dataPath + N'InMemoryBoxDemo.mdf'', size=102400KB, filegrowth = 102400KB),
filegroup [HKData] contains memory_optimized_data (name=N''InMemoryBoxDemo_HekatonData'', filename=N''' + @HKPath + N''')
log on (name=N''InMemoryBoxDemo_log'', filename=N''' + @logPath + N'InMemoryBoxDemo.ldf'', size=256000KB, filegrowth = 256000KB);

alter database [InMemoryBoxDemo] set recovery simple;'

raiserror('Creating database InMemoryBoxDemo',0,1) with nowait
raiserror('Data Path: %s',0,1,@dataPath) with nowait
raiserror('Log Path: %s',0,1,@logPath) with nowait
raiserror('Hekaton Folder: %s',0,1,@HKPath) with nowait
raiserror('Statement:',0,1) with nowait
raiserror(@sql,0,1) with nowait

exec sp_executesql @sql
go

